@@ -32,6 +32,18 @@
 #endif
 
 
+#ifndef HAVE_RB_THREAD_FD_SELECT
+#define rb_fdset_t fd_set
+#define rb_fd_isset(n, f) FD_ISSET(n, f)
+#define rb_fd_init(f) FD_ZERO(f)
+#define rb_fd_zero(f)  FD_ZERO(f)
+#define rb_fd_set(n, f)  FD_SET(n, f)
+#define rb_fd_clr(n, f) FD_CLR(n, f)
+#define rb_fd_term(f)
+#define rb_thread_fd_select rb_thread_select
+#endif
+
+
 #include <ruby.h>
 #include <string.h>
 #include <math.h>
@@ -99,13 +111,14 @@ VALUE do_postgres_typecast(const char *value, long length, const VALUE type, int
 }
 
 void do_postgres_raise_error(VALUE self, PGresult *result, VALUE query) {
-  const char *message = PQresultErrorMessage(result);
+  VALUE message = rb_str_new2(PQresultErrorMessage(result));
   char *sql_state = PQresultErrorField(result, PG_DIAG_SQLSTATE);
   int postgres_errno = MAKE_SQLSTATE(sql_state[0], sql_state[1], sql_state[2], sql_state[3], sql_state[4]);
+  VALUE str = rb_str_new2(sql_state);
 
   PQclear(result);
 
-  data_objects_raise_error(self, do_postgres_errors, postgres_errno, message, query, rb_str_new2(sql_state));
+  data_objects_raise_error(self, do_postgres_errors, postgres_errno, message, query, str);
 }
 
 /* ====== Public API ======= */
@@ -269,12 +282,12 @@ PGresult * do_postgres_cCommand_execute_async(VALUE self, VALUE connection, PGco
   }
 
   int socket_fd = PQsocket(db);
-  fd_set rset;
+  rb_fdset_t rset;
 
   while (1) {
-    FD_ZERO(&rset);
-    FD_SET(socket_fd, &rset);
-    retval = rb_thread_select(socket_fd + 1, &rset, NULL, NULL, NULL);
+    rb_fd_init(&rset);
+    rb_fd_set(socket_fd, &rset);
+    retval = rb_thread_fd_select(socket_fd + 1, &rset, NULL, NULL, NULL);
 
     if (retval < 0) {
       rb_sys_fail(0);
